<?php
namespace Financeiro\Logic\ContasReceber\Cnab\Remessa;

use \Geral\Entities\Banco;
use \Geral\Entities\Municipio;
use \Geral\Entities\Endeclie;
use \Geral\Entities\Cliente;
use \Financeiro\Entities\Boletreceber;
use \Financeiro\Entities\Emprbanc;
use \Geral\Util\DateUtil;
use \Financeiro\Entities\Contareceber;
use \Geral\Entities\Empresa;
use \Financeiro\Util\Util;

class GeraCnabSantander extends GeraCnab{
    
const TIPO_INSCRICAO_CEDENTE_CNPJ = '2';
const TIPO_INSCRICAO_CEDENTE_CPF = '1';
const CODIGO_ARQUIVO = '1';
const VERSAO_ARQUIVO = '040';
const LOTE_SERVICO = '0000';
const LOTE_SERVICO_2 = '0001';
const IDENTIFICACAO_ARQUIVO = '0';
const NUMERO_LOTE = '0001';
const TIPO_REGISTRO = '1';
const TIPO_OPERACAO = 'R';
const TIPO_SERVICO = '01';
const VERSAO_ARQUIVO_LOTE = '030';
const IDENTIFICACAO_REGISTRO = '3';
const CODIGO_SEGMENTO = 'P';
const ID_OPERACAO_CNAB_INCLUSAO = '01';
const ID_OPERACAO_CNAB_EXCLUSAO = '02';
const ID_OPERACAO_CNAB_ALTERACAO = '06';
const TIPO_COBRANCA_SIMPLES_SREGISTRO = '1';
const TIPO_COBRANCA_SIMPLES_CREGISTRO = '5';
const TIPO_COBRANCA_SIMPLES_CAUSIONADA = '3';
const TIPO_COBRANCA_SIMPLES_CAUSIONADA_RAPIDA = '6';
const FORMA_CADASTRAMENTO = '1';
const TIPO_DOCUMENTO = '2';
const CODIGO_DESCONTO = '0';
const CODIGO_BAIXA = '2';
const ESPECIE_TITULO = '02';
const IDTITULO_ACEITO = 'N';	
const CODIGO_MOEDA = '00';
const CODIGO_SEGMENTO_Q = 'Q';
const CODIGO_SEGMENTO_R = 'R';
const CODIGO_DESCONTO_2 = '0';
const CODIGO_MULTA_1 = '1';

const CODIGO_MULTA_2 = '2';
const IDENTIFICACAO_ARQUIVO_TRAILLER_LOTE = '5';
const IDENTIFICACAO_ARQUIVO_TRAILLER = '9';
const LOTE_SERVICO_TRAILLER = '9999';

public function geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnab, $cdreducont, $carteira = NULL){

    if (!is_dir($caminho))
        mkdir($caminho, "0777", true);
    $nmarquivo = $caminho.$nmarquivo;
    $file = fopen($nmarquivo, "w+");

    tk_set_env_var('REGISTROS', tk_get_env_var('REGISTROS') + 1);

    // CAMPO: Código do Banco
    // POSIÇÃO: 001 a 003
    // PICTURE: 9(3)
    /**@var $banco Banco*/
    $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
    $arquivo = Util::campoCnabValido($banco->getCdbanccnab());

    // CAMPO: Lote de Serviço
    // POSIÇÃO: 004 a 007
    // PICTURE: 9(4)
    $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO);

    // CAMPO: Identificação do Arquivo
    // POSIÇÃO: 008 a 008
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_ARQUIVO);

    // CAMPO: Uso exclusivo FEBRABAN
    // POSIÇÃO: 009 a 016
    // PICTURE: 008
    $arquivo = str_pad($arquivo, 16);

    // CAMPO: Tipo da inscrição da empresa
    // POSIÇÃO: 017 a 017
    // PICTURE: 9(1)
    /**@var $nrinsjurempr Empresa*/
    $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
    $idtpijurempr = $nrinsjurempr->getNrinsjurempr();
    if (strlen($idtpijurempr) == '14'){
       $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CNPJ; //CNPJ
    }
    else{
       $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CPF; //CPF
    }
    $arquivo .= Util::campoCnabValido($stTpInscCedente);

    // CAMPO: Nº de inscrição da empresa
    // POSIÇÃO: 018 a 032
    // PICTURE: 9(15)
    /**@var $nrinsjurempr Empresa*/
    $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
    $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr->getNrinsjurempr(), 15, "0", STR_PAD_LEFT));

    //Campo: Código de transmissão
    //POSIÇÃO: 33 a 47
    //PICTURE: 9(15)
    //Busca o identificador da conta
    //se estiver vazio: busca identificador do Banco
    //se ambos estiverem vazios, grava zeros.
    /** @var $emprbanc Emprbanc*/
    $emprbanc = Emprbanc::findOneByNrorgAndCdempresaAndCdbanco($nrorg, $cdempresa, $cdbanco);
    if((bool)$cdidrec === true){
        $cdidreccontcorr = str_pad($cdidrec, 15);
        $arquivo .= Util::campoCnabValido($cdidreccontcorr);
    }
    else if((bool)$emprbanc->getCdempbancrec() === true){
        $cdempbancrec = str_pad($emprbanc->getCdempbancrec(), 15);
        $arquivo .= Util::campoCnabValido($cdempbancrec);
    }
    else
        $arquivo = str_pad($arquivo, 47, "0");

    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 048 - 072
    // PICTURE: 025
    $arquivo = str_pad($arquivo, 72);

    // CAMPO: Nome da Empresa
    // POSIÇÃO: 073 a 0102
    // PICTURE: X(30)
    /**@var $nomeempresa Empresa*/
    $nomeempresa = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
    $nmempresa = substr($nomeempresa->getNmempresa(), 0, 31);
    $arquivo .= Util::campoCnabValido(str_pad($nmempresa, 30));

    // CAMPO: Nome do Banco
    // POSIÇÃO: 103 a 132
    // PICTURE: X(30)
    /** @var $nmbanco Banco*/
    $nmbanco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
    $arquivo .= Util::campoCnabValido(substr($nmbanco->getNmbanco(), 0, 31));

    // CAMPO: Brancos
    // POSIÇÃO: 133 a 142
    // PICTURE: X(10)
    $arquivo = str_pad($arquivo, 142);

    // CAMPO: Código do Arquivo
    // POSIÇÃO: 143 a 143
    // PICTURE: X(1)
    $arquivo .= Util::campoCnabValido('1'); // REMESSA

    // CAMPO: Data de Geração do arquivo
    // POSIÇÃO: 144 a 151
    // PICTURE: 9(8)
    $now = DateUtil::getDataDeString(DateUtil::truncateString(now()));
    $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
    $arquivo .= Util::campoCnabValido($data);

    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 152 - 157
    // PICTURE: 9(6)
    $arquivo = str_pad($arquivo, 157);

    // CAMPO: Número Seqüencial
    // POSIÇÃO: 158 a 163
    // PICTURE: 9(6)
    $arquivo .= Util::campoCnabValido(substr($nrseqremessarec, -6));

    // CAMPO: Versão do Arquivo
    // POSIÇÃO: 164 a 166
    // PICTURE: X(3)
    $arquivo .= Util::campoCnabValido(self::VERSAO_ARQUIVO);

    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 167 - 240
    // PICTURE: 074
    $arquivo = str_pad($arquivo, 240) . "\r\n";

    //Grava a variavel arquivo no arquivo texto
    fprintf($file, "%s", $arquivo);
    fclose($file);
}

public function geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec, $cdreducont, $carteira = NULL){

    $nmarquivo = $caminho.$nmarquivo;
    $file = fopen($nmarquivo, "a+");
    tk_set_env_var('REGISTROS', tk_get_env_var('REGISTROS') + 1);

    // CAMPO: Código do Banco
    // POSIÇÃO: 001 a 003
    // PICTURE: 9(3)
    /**@var $banco Banco*/
    $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
    $arquivo = $banco->getCdbanccnab();

    // CAMPO: Lote de Serviço
    // POSIÇÃO: 004 a 007
    // PICTURE: 9(4)
    $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_2);

    // CAMPO: Tipo de Registro
    // POSIÇÃO: 008 a 008
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO);

    // CAMPO: Tipo de Operação
    // POSIÇÃO: 009 a 009
    // PICTURE: X(1)
    $arquivo .= Util::campoCnabValido(self::TIPO_OPERACAO);

    // CAMPO: Tipo de Serviço
    // POSIÇÃO: 010 a 011
    // PICTURE: 9(2)
    $arquivo .= Util::campoCnabValido(self::TIPO_SERVICO);

    // CAMPO: Forma de Lançamento
    // POSIÇÃO: 012 a 013
    // PICTURE: X(2)
    $arquivo = str_pad($arquivo, 13);

    // CAMPO: Versão do Arquivo
    // POSIÇÃO: 014 a 016
    // PICTURE: X(3)
    $arquivo .= Util::campoCnabValido(self::VERSAO_ARQUIVO_LOTE);

    // CAMPO: Brancos
    // POSIÇÃO: 017 a 017
    // PICTURE: 9(1)
    $arquivo = str_pad($arquivo, 17);

    // CAMPO: Tipo da inscrição da empresa
    // POSIÇÃO: 018 a 018
    // PICTURE: 9(1)        
    /**@var $nrinsjurempr Empresa*/
    $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
    $idtpijurempr = $nrinsjurempr->getNrinsjurempr();
    if (strlen($idtpijurempr) == '14'){
        $stTpInscCedente = '2'; //CNPJ
    }
    else{
        $stTpInscCedente = '1'; //CPF
    }
    $arquivo .= Util::campoCnabValido($stTpInscCedente);

    // CAMPO: Nº de inscrição da empresa
    // POSIÇÃO: 018 a 032
    // PICTURE: 9(15)
    /**@var $nrinsjurempr Empresa*/
    $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
    $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr->getNrinsjurempr(), 15, "0", STR_PAD_LEFT));

    // CAMPO: Uso Exclusivo do Banco
    // POSIÇÃO: 034 a 053
    // PICTURE: X(20)
    $arquivo = str_pad($arquivo, 53);

    //CAMPO: Código de transmissão
    //POSIÇÃO: 54 a 68
    //PICTURE: 015
    //Busca o identificador da conta
    //se estiver vazio: busca identificador do Banco
    //se ambos estiverem vazios, grava zeros.
    /** @var $emprbanc Emprbanc*/
    $emprbanc = Emprbanc::findOneByNrorgAndCdempresaAndCdbanco($nrorg, $cdempresa, $cdbanco);
    if((bool)$cdidrec === true){
        $arquivo .= Util::campoCnabValido($cdidrec);
    }
    else if((bool)$emprbanc->getCdempbancrec() === true){
        $arquivo .= Util::campoCnabValido($emprbanc->getCdempbancrec());
    }
    else
        $arquivo = str_pad($arquivo, 68, "0");

    // CAMPO: Uso Exclusivo do Banco
    // POSIÇÃO: 069 - 073
    // PICTURE: 005
    $arquivo = str_pad($arquivo, 73);

    // CAMPO: Nome da Empresa
    // POSIÇÃO: 074 a 0103
    // PICTURE: X(30)
    /**@var $nomeempresa Empresa*/
    $nomeempresa = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
    $nmempresa = str_pad($nomeempresa->getNmempresa(), 30);
    $arquivo .= Util::campoCnabValido(substr($nmempresa, 0, 31));

    // CAMPO: Mensagem 01
    // POSIÇÃO: 104 a 143
    // PICTURE: X(40)
    $arquivo = str_pad($arquivo, 143);

    // CAMPO: Mensagem 02
    // POSIÇÃO: 144 a 183
    // PICTURE: X(40)
    $arquivo = str_pad($arquivo, 183);

    // CAMPO: Número REMESSA/RETORNO
    // POSIÇÃO: 184 a 191
    // PICTURE: 9(8)
    $arquivo .= Util::campoCnabValido(substr($nrcnabrec, -8));

    // CAMPO: Data de Gravação REMESSA/RETORNO
    // POSIÇÃO: 192 a 199
    // PICTURE: 9(8)
    $now = DateUtil::getDataDeString(DateUtil::truncateString(now()));
    $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
    $arquivo .= Util::campoCnabValido($data);

    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 200 - 240
    // PICTURE: 041
    $arquivo = str_pad($arquivo, 240) . "\r\n";;

    //Grava a variavel arquivo no arquivo texto
    fprintf($file, "%s", $arquivo);
    fclose($file);
}

public function geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec,
                                     $nrcnabrec, $idoperacao, $cdagenciacnab, $cdcontacorrente, $cdcontcorrcobranca, $nossonumero,
                                     $idtipocobranca, $nrlanbolrec, $cdfilial, $dtvenbolrec, $valorfinalboleto, $valorpemobrec, $valormordiabrec,
                                     $valorpemulbrec, $valormultabrec, $valorfinalbrec, $cdcartcobbol, $cddiaprotbol, $cdreducont, $imprbol,
                                     $qtdiaprotbol){

    $nmarquivo = $caminho.$nmarquivo;
    $file = fopen($nmarquivo, "a+");
    tk_set_env_var('REGISTROS', tk_get_env_var('REGISTROS') + 1);

    // CAMPO: Código do Banco
    // POSIÇÃO: 001 a 003
    // PICTURE: 9(3)
    /**@var $banco Banco*/
    $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
    $arquivo = $banco->getCdbanccnab();

    // CAMPO: Lote de Serviço
    // POSIÇÃO: 004 a 007
    // PICTURE: 9(4)
    $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_2);

    // CAMPO: Identificação do registro
    // POSIÇÃO: 008 a 008
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_REGISTRO);

    // CAMPO: Nº seq. do reg. no lote
    // POSIÇÃO: 009 a 013
    // PICTURE: 9(5)
    // Número de seqüencia do registro no lote inicializado sempre em 1
    $contador = $contador = tk_get_env_var('CONTADOR');
    $contador++;
    $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
    tk_set_env_var('CONTADOR', $contador);
    $arquivo .= Util::campoCnabValido($contador);

    // CAMPO: Código do Segmento
    // POSIÇÃO: 014 a 014
    // PICTURE: X(1)
    $arquivo .= Util::campoCnabValido(self::CODIGO_SEGMENTO);

    // CAMPO: Brancos
    // POSIÇÃO: 015 a 015
    // PICTURE: X(1)
    $arquivo = str_pad($arquivo, 15);

    // CAMPO: Código do Movimento
    // POSIÇÃO: 016 a 017
    // PICTURE: 9(2)
    $idopecnabrec = $idoperacao;
    if((bool) $idopecnabrec !== false){
          if($idopecnabrec == 'I')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_INCLUSAO); //Inclusão
          else if($idopecnabrec == 'E')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_EXCLUSAO); //Exclusão
          else
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_ALTERACAO); //Alteração
    }

    // CAMPO: Agência do Cedente  + // CAMPO: Dígito da Agência do Cedente
    // POSIÇÃO: 018 - 021          + // POSIÇÃO: 022 - 022
    // PICTURE: 004               + // PICTURE: 001
    $cdagencia = $cdagenciacnab;
    if((bool) $cdagencia !== false){
        $arquivo .= Util::campoCnabValido($cdagencia);
    }
    else
        $arquivo = str_pad($arquivo, 22, "0");

    // CAMPO: Número da conta corrente
    // POSIÇÃO: 023 - 031
    // PICTURE: 009
    // MAIS
    // CAMPO: Dígito verificador da conta
    // POSIÇÃO: 032 - 032
    // PICTURE: 001
    $cdcontcorr = $cdcontacorrente;
    if((bool) $cdcontcorr !== false){
        $arquivo .= Util::campoCnabValido(str_pad($cdcontcorr, 10, "0", STR_PAD_LEFT));
    }
    else
        $arquivo = str_pad($arquivo, 32, "0");

    // CAMPO: Conta cobrança
    // POSIÇÃO: 033 - 041
    // PICTURE: 009
    //MAIS
    // CAMPO: Dígito da conta cobraNºa
    // POSIÇÃO: 042 - 042
    // PICTURE: 001
    $cdcontcorrcobr = $cdcontcorrcobranca;
    if((bool) $cdcontcorrcobr !== false){
        $arquivo .= Util::campoCnabValido(str_pad($cdcontcorrcobr, 10, "0", STR_PAD_LEFT));
    }
    else
        $arquivo = str_pad($arquivo, 42, "0");

    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 043 - 044
    // PICTURE: 002
    $arquivo = str_pad($arquivo, 44);

    // CAMPO: Nosso Número
    // POSIÇÃO: 045 a 057
    // PICTURE: 013
    // preferencial [Cobrança registrada]
    $cdnosnumrec = $nossonumero;
    if((bool) $cdnosnumrec !== false){
        $arquivo .= str_replace(' ', '', Util::campoCnabValido($cdnosnumrec));
    }
    else
        $arquivo = str_pad($arquivo, 57, "0");

    // CAMPO: Tipo de cobrança
    // POSIÇÃO: 058 - 058
    // PICTURE: 001
    //Verifica o Tipo de Cobrança
    //1 - Cobrança Simples (Sem Registro e Eletrônica com Registro) 3 - Cobrança Causionada (Eletrônica com registro e Convencional com Registro)
    //5 - Cobrança Sinples (Com Registro) 6 - Cobrança Caucionada (Rápida com Registro)
    $idcartcobrec = $idtipocobranca;
    if((bool) $idcartcobrec !== false){
          switch($idcartcobrec){
                  case 1:
                          $arquivo .= Util::campoCnabValido(self::TIPO_COBRANCA_SIMPLES_SREGISTRO);
                          break;
                  case 2:
                          $arquivo .= Util::campoCnabValido(self::TIPO_COBRANCA_SIMPLES_CREGISTRO);
                          break;
                  case 3:
                          $arquivo .= Util::campoCnabValido(self::TIPO_COBRANCA_SIMPLES_CAUSIONADA);
                          break;
                  case 4:
                          $arquivo .= Util::campoCnabValido(self::TIPO_COBRANCA_SIMPLES_CAUSIONADA_RAPIDA);
                          break;			
          }  
    }

    // CAMPO: Forma de Cadastramento
    // POSIÇÃO: 059 a 059
    // PICTURE: 9 (001)
    // 1 - Cobrança Registrada 2 - Cobrança sem Registro
    $arquivo .= Util::campoCnabValido(self::FORMA_CADASTRAMENTO);

    // CAMPO: Documento - Tipo de documento
    // POSIÇÃO: 060 a 060
    // PICTURE: 9 (001)
    $arquivo .= Util::campoCnabValido(self::TIPO_DOCUMENTO);

    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 061 a 061
    // PICTURE: X(1)
    $arquivo = str_pad($arquivo, 61);


    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 062 a 062
    // PICTURE: X(1)
    $arquivo = str_pad($arquivo, 62);

    // CAMPO: Identificação do título na empresa
    // POSIÇÃO: 063 a 077
    // PICTURE: X(15)
    /** @var $contareceber \Financeiro\Repositories\Contareceber */
    $contareceber = Contareceber::getRepository();
    $duplicata = $contareceber->consultaNumeroDuplicata($nrlanbolrec, $cdfilial, $nrorg);
    $sql = '';
    foreach($duplicata as $duplicatas){
          $sql = str_pad($duplicatas['DUPLICATA'] . $duplicatas['NRPARCELA'], 10, '0', STR_PAD_LEFT);
    }
    $arquivo .= Util::campoCnabValido(str_pad($sql, 15));

    // CAMPO: Data de vencimento do título
    // POSIÇÃO: 078 a 085
    // PICTURE: 9(8)
    $vencimento = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
    $vencimento = DateUtil::getDia($vencimento) . DateUtil::getMes($vencimento) . DateUtil::getAno($vencimento);
    $arquivo .= Util::campoCnabValido($vencimento);

    // CAMPO: Valor do título
    // POSIÇÃO: 086 a 100
    // PICTURE: 9(13)V9(2)
    $vrfinabolrec = str_pad(($valorfinalboleto * 100), 15, "0", STR_PAD_LEFT);
    $arquivo .= Util::campoCnabValido($vrfinabolrec);

    // CAMPO: Agência encarregada da cobrança
    // POSIÇÃO: 101 - 104
    // PICTURE: 004
    // CAMPO: Dígito da Agência do Cedente
    // POSIÇÃO: 105 - 105
    // PICTURE: 001
    //Em caso de protesto preencher com zeros
    $cdagencia = str_pad($cdagenciacnab, 5);
    if($cddiaprotbol != '3')
        $arquivo = str_pad($arquivo, 105, "0");

    elseif((bool) $cdagencia !== false)
        $arquivo .= Util::campoCnabValido($cdagencia);

    else
        $arquivo = str_pad($arquivo, 105, "0");

    // CAMPO: Uso Exclusivo do Banco
    // POSIÇÃO: 106 a 106
    // PICTURE: X(1)
    $arquivo = str_pad($arquivo, 106);

    // CAMPO: Espécie do Título
    // POSIÇÃO: 107 a 108
    // PICTURE: 9(2)
    $arquivo .= Util::campoCnabValido(self::ESPECIE_TITULO);

    // CAMPO: Identificação de título aceito ou não aceito
    // POSIÇÃO: 109 a 109
    // PICTURE: X(1)		  
    $arquivo .= Util::campoCnabValido(self::IDTITULO_ACEITO);

    // CAMPO: Data de emissão do título
    // POSIÇÃO: 110 a 117
    // PICTURE: 9(8)
    $now = DateUtil::getDataDeString(DateUtil::truncateString(now()));
    $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
    $arquivo .= Util::campoCnabValido($data);

    // CAMPO: Código de Juros de Mora
    // POSIÇÃO: 118 a 118
    // PICTURE: 9(1)
    // CAMPO: Data de Juros de Mora
    // POSIÇÃO: 119 a 126
    // PICTURE: 9(8)
    // CAMPO: Valor de mora por dia de atraso
    // POSIÇÃO: 127 a 141
    // PICTURE: 9(13)V9(2)
    $vrpemorbrec = $valorpemobrec;
    $vrmordiabrec = $valormordiabrec;
    $vrpemulbrec = $valorpemulbrec;
    $vrmultabrec = $valormultabrec;

    if (($vrpemorbrec == 0) or ((bool) $vrpemorbrec == false)){
          if (($vrpemulbrec == 0) or ((bool) $vrpemulbrec == false)){
                  if (($vrmordiabrec == 0) or ((bool) $vrmordiabrec == false)){
                          if (($vrmultabrec == 0) or ((bool) $vrmultabrec == false)){
                              $arquivo .= Util::campoCnabValido('3');
                              $arquivo = str_pad($arquivo, 126, "0");
                              $arquivo = str_pad($arquivo, 141, "0");
                          }
                          else{
                              $arquivo .= Util::campoCnabValido('2');
                              $arquivo .= Util::campoCnabValido($vencimento);
                              $vr = ($valormultabrec * 100 / $valorfinalbrec * 30) * 100000;
                              $vr = str_pad($vr, 15, "0", STR_PAD_LEFT);
                              $arquivo .= Util::campoCnabValido($vr);
                          }
                  }
                  else{
                      $arquivo .= Util::campoCnabValido('2');
                      $arquivo .= Util::campoCnabValido($vencimento);
                      $vr = ($vrpemulbrec * 30) * 100000;
                      $vr = str_pad($vr, 15, "0", STR_PAD_LEFT);
                      $arquivo .= Util::campoCnabValido($vr);
                  }
          }
          else{
              $arquivo .= Util::campoCnabValido('1');
              $arquivo .= Util::campoCnabValido($vencimento);
              $vr = $vrmordiabrec * 100;
              $vr = str_pad($vr, 15, "0", STR_PAD_LEFT);
              $arquivo .= Util::campoCnabValido($vr);
          }
    }
    else{
        $arquivo .= Util::campoCnabValido('2');
        $arquivo .= Util::campoCnabValido($vencimento);
        $vr = $vrpemorbrec * 100000;
        $vr = str_pad($vr, 15, '0', STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($vr);
    }


    // CAMPO: Código do Desconto
    // POSIÇÃO: 142 a 142
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido(self::CODIGO_DESCONTO);

    // CAMPO: Data de Desconto
    // POSIÇÃO: 143 a 150
    // PICTURE: 9(8)
    $arquivo = str_pad($arquivo, 150, "0");

    // CAMPO: Valor do Desconto
    // POSIÇÃO: 151 a 165
    // PICTURE: 9(13)V9(2)
    $arquivo = str_pad($arquivo, 165, "0");

    // CAMPO: Valor do IOF a ser recolhido
    // POSIÇÃO: 166 a 180
    // PICTURE: 9(13)V9(2)
    $arquivo = str_pad($arquivo, 180, "0");

    // CAMPO: Valor do Abatimento
    // POSIÇÃO: 181 a 195
    // PICTURE: 9(13)V9(2)
    $arquivo = str_pad($arquivo, 195, "0");

    // CAMPO: Identificação do Título na Empresa
    // POSIÇÃO: 196 a 220
    // PICTURE: X(25)
    $arquivo .= Util::campoCnabValido(str_pad($cdfilial, 4) . str_pad($nrlanbolrec, 10) . 'CART' . str_pad($idtipocobranca.$cdcartcobbol, 7));

    /** @var $boletreceber Boletreceber */
    $stcddiaprot = '3';
    $stqtdiaprot = '00';
    if((bool) $nrlanbolrec !== false){
            $boletreceber = Boletreceber::findOneByNrorgAndNrlanbolrec($nrorg, $nrlanbolrec);
            if((bool) $boletreceber->getCddiaprotbol() === false){
                  $stcddiaprot = '3';
            }
            else{
                  $stcddiaprot = $boletreceber->getCddiaprotbol();
            }
            if((bool) $boletreceber->getQtdiaprotbol() === false){
                  $stqtdiaprot = '00';
            }
            else{
                  $stqtdiaprot = $boletreceber->getQtdiaprotbol();
                          if($stqtdiaprot != '00')
                                  $stcddiaprot = '1';
            }
            if (($stcddiaprot = '3') or ($stqtdiaprot = '00')){
                  $stcddiaprot = '3';
                  $stqtdiaprot = '00';
            }
    }

    // CAMPO: Código para Protesto
    // POSIÇÃO: 221 a 221
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido($stcddiaprot);

    // CAMPO: Prazo para Protesto
    // POSIÇÃO: 222 a 223
    // PICTURE: 9(2)
    $arquivo .= Util::campoCnabValido($stqtdiaprot);

    // CAMPO: Código para Baixa/Devolução
    // POSIÇÃO: 224 a 224
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido(self::CODIGO_BAIXA);

    // CAMPO: Número de dias para Baixa/Devolução
    // POSIÇÃO: 225 a 227
    // PICTURE: 9(3)
    $arquivo = str_pad($arquivo, 227, "0");

    // CAMPO: Código da moeda
    // POSIÇÃO: 228 a 229
    // PICTURE: 9(2)
    $arquivo .= Util::campoCnabValido(self::CODIGO_MOEDA);

    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 230 -240
    // PICTURE: 011
    $arquivo = str_pad($arquivo, 240) . "\r\n";

    fprintf($file, "%s", $arquivo);
    fclose($file);
}

public function geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $cdcliente, $idoperacao, $cdempresa, $cdfilial, $nrlanbolrec){

    //################  GRAVA O ENDEREÇO DO CLIENTE ###################//
    $nmarquivo = $caminho.$nmarquivo;
    $file = fopen($nmarquivo, "a+");
    tk_set_env_var('REGISTROS', tk_get_env_var('REGISTROS') + 1);

    // CAMPO: Código do Banco
    // POSIÇÃO: 001 a 003
    // PICTURE: 9(3)
    /**@var $banco Banco*/
    $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
    $arquivo = $banco->getCdbanccnab();

    // CAMPO: Lote de Serviço
    // POSIÇÃO: 004 a 007
    // PICTURE: 9(4)
    $arquivo .= Util::campoCnabValido(self::NUMERO_LOTE);

    // CAMPO: Identificação do registro
    // POSIÇÃO: 008 a 008
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_REGISTRO);

    // CAMPO: Nº seq. do reg. no arq.
    // POSIÇÃO: 009 a 013
    // PICTURE: 9(5)
    // Número de seqüência do registro no lote inicializado sempre em 1
    $contador = tk_get_env_var('CONTADOR');
    $contador++;
    $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
    tk_set_env_var('CONTADOR', $contador);
    $arquivo .= Util::campoCnabValido($contador);

    // CAMPO: Código do Segmento
    // POSIÇÃO: 014 a 014
    // PICTURE: X(1)
    $arquivo .= Util::campoCnabValido(self::CODIGO_SEGMENTO_Q);

    // CAMPO: Brancos
    // POSIÇÃO: 015 a 015
    // PICTURE: X(1)
    $arquivo = str_pad($arquivo, 15);

    // CAMPO: Código do Movimento
    // POSIÇÃO: 016 a 017
    // PICTURE: 9(2)
    $idopecnabrec = $idoperacao;
    if((bool) $idopecnabrec !== false){
          if($idopecnabrec == 'I')
                  $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_INCLUSAO); //Inclusão
          else if($idopecnabrec == 'E')
                  $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_EXCLUSAO); //Exclusão
          else
                  $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_ALTERACAO); //Alteração
    }

    // CAMPO: Código de inscrição/Sacado ('1' = CPF, '2' = CNPJ)
    // POSIÇÃO: 018 a 018
    // PICTURE: 9(1)
    /**@var $nrinsjurempr Empresa*/
    $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
    $idtpijurempr = $nrinsjurempr->getNrinsjurempr();
    if (strlen($idtpijurempr) == '14'){
       $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CNPJ; //CNPJ
    }
    else{
       $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CPF; //CPF
    }
    $arquivo .= Util::campoCnabValido($stTpInscCedente);

    // CAMPO: Nº de inscrição do sacado (CPF/CGC)
    // POSIÇÃO: 019 a 033
    // PICTURE: 9(15)
    /** @var $cliente Cliente */
    $cliente = Cliente::findOneByNrorgAndCdcliente($nrorg, $cdcliente);
    if((bool)$cdcliente === true){
        $nrinsjurclie = Util::removeMascaraInscricao($cliente->getNrinsjurclie());
    }
    if((bool) $nrinsjurclie !== false){
        $arquivo .= str_pad($nrinsjurclie, 15, "0", STR_PAD_LEFT);
    }
    else{
        $arquivo = str_pad($arquivo, 33, "0");
    }

    // CAMPO: Nome do sacado
    // POSIÇÃO: 034 a 073
    // PICTURE: X(40)
    if($cdcliente){
    	$copia_nmrazsoc = $cliente->getNmrazsocclie();
    	$nmrazsocclie = Util::campoCnabValido($copia_nmrazsoc);
        $arquivo .= str_pad(substr($nmrazsocclie, 0, 40) ,40);
    }

    else{
        $arquivo = str_pad($arquivo, 73, "0");
    }

    // CAMPO: Logradouro (Endereço do sacado (cliente))
    // POSIÇÃO: 074 a 113
    // PICTURE: X(40)
    /** @var $endeclie Endeclie */
    $endeclie = Endeclie::findOneByNrorgAndCdclienteAndIdtpendeclie($nrorg, $cdcliente, 'C');
    $dsendeclie = Util::campoCnabValido($endeclie->getDsendeclie());
    if((bool) $dsendeclie !== false){
        $arquivo .= str_pad(substr($dsendeclie, 0, 40), 40);
    }
    else{
        $arquivo = str_pad($arquivo, 113, "0");
    }

    // CAMPO: Bairro do sacado
    // POSIÇÃO: 114 a 128
    // PICTURE: X(15)
    $nmbairclie = str_pad($endeclie->getNmbairclie(), 15);
    if((bool) $nmbairclie !== false){
        $arquivo .= Util::campoCnabValido(substr($nmbairclie, 0, 16));
    }
    else{
        $arquivo = str_pad($arquivo, 128, "0");
    }

    // CAMPO: Cep sacado
    // POSIÇÃO: 129 - 133
    // PICTURE: 005

    // CAMPO: Sufixo do Cep do sacado
    // POSIÇÃO: 134 - 136
    // PICTURE: 003
    $nrcepclie = str_pad(str_replace(" ", null, $endeclie->getNrcepclie()), 8, "0");
    if((bool) $nrcepclie !== false){
        $arquivo .= Util::campoCepTel($nrcepclie);
    }
    else{
        $arquivo = str_pad($arquivo, 136, "0");
    }

    // CAMPO: Cidade do sacado
    // POSIÇÃO: 137 a 151
    // PICTURE: X(15)
    /** @var $municipio Municipio */
    $municipio = Municipio::findOneByNrorgAndCdmunicipio($nrorg, $endeclie->getCdmunicipio());
    $nmmunicipio = substr($municipio->getNmmunicipio(), 0, 16);
    if((bool) $nmmunicipio !== false){
        $arquivo .= Util::campoCnabValido(str_pad($nmmunicipio, 15));
    }
    else{
        $arquivo = str_pad($arquivo, 151, "0");
    }

    // CAMPO: Estado do sacado
    // POSIÇÃO: 152 a 153
    // PICTURE: X(2)
    $sgestado = str_pad($endeclie->getSgestado(), 2);
    if((bool) $sgestado !== false){
        $arquivo .= Util::campoCnabValido($sgestado);
    }
    else{
        $arquivo = str_pad($arquivo, 154, "0");
    }

    // CAMPO: Código de inscrição do sacador ou avalista
    // POSIÇÃO: 154 a 154
    // PICTURE: 9(1)
    $arquivo = str_pad($arquivo, 154, "0");

    // CAMPO: Nº de inscrição do sacador ou avalista
    // POSIÇÃO: 155 a 169
    // PICTURE: 9(15)
    $arquivo = str_pad($arquivo, 169, "0");

    // CAMPO: Nome do sacador ou avalista
    // POSIÇÃO: 170 a 209
    // PICTURE: X(40)
    $arquivo = str_pad($arquivo, 209);

    // CAMPO: Identificador de carne
    // POSIÇÃO: 210 a 212
    // PICTURE: X(3)
    $arquivo = str_pad($arquivo, 212, "0");

    // CAMPO: Sequencial da Parcela ou Número inicial da parcela
    // POSIÇÃO: 213 -215
    // PICTURE: 003
    $arquivo = str_pad($arquivo, 215, "0");

    // CAMPO: Quantidade total de parcelas
    // POSIÇÃO: 216 -218
    // PICTURE: 003
    $arquivo = str_pad($arquivo, 218, "0");

    // CAMPO: Número do plano
    // POSIÇÃO: 219 - 221
    // PICTURE: 003
    $arquivo = str_pad($arquivo, 221, "0");

    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 222 - 240
    // PICTURE: 019
    $arquivo = str_pad($arquivo, 240) . "\r\n";

    fprintf($file, "%s", $arquivo);
    fclose($file);
}

public function geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec){

    $nmarquivo = $caminho.$nmarquivo;
    $file = fopen($nmarquivo, "a+");
    tk_set_env_var('REGISTROS', tk_get_env_var('REGISTROS') + 1);

    // CAMPO: Código do Banco
    // POSIÇÃO: 001 a 003
    // PICTURE: 9(3)
    /**@var $banco Banco */
    $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
    $arquivo = $banco->getCdbanccnab();

    // CAMPO: Lote de Serviço
    // POSIÇÃO: 004 a 007
    // PICTURE: 9(4)
    $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_2);

    // CAMPO: Identificação do registro
    // POSIÇÃO: 008 a 008
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_REGISTRO);

    // CAMPO: Nº seq. do reg. no arq.
    // POSIÇÃO: 009 a 013
    // PICTURE: 9(5)
    $contador = tk_get_env_var('CONTADOR');
    $contador++;
    $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
    tk_set_env_var('CONTADOR', $contador);
    $arquivo .= Util::campoCnabValido($contador);


    // CAMPO: Código do Segmento
    // POSIÇÃO: 014 a 014
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido(self::CODIGO_SEGMENTO_R);

    // CAMPO: Brancos
    // POSIÇÃO: 015 a 015
    // PICTURE: X(1)
    $arquivo = str_pad($arquivo, 15);

    // CAMPO: Código de Movimento
    // POSIÇÃO: 016 a 017
    // PICTURE: 9(2)
    $idopecnabrec = $idoperacao;
    if((bool) $idopecnabrec !== false){
        if($idopecnabrec == 'I')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_INCLUSAO); //Inclusão
        else if($idopecnabrec == 'E')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_EXCLUSAO); //Exclusão
        else
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_ALTERACAO); //Alteração
    }

    // CAMPO: Código de desconto 2
    // POSIÇÃO: 018 a 018
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido(self::CODIGO_DESCONTO_2);

    // CAMPO: Data do desconto 2
    // POSIÇÃO: 019 a 026
    // PICTURE: 9(8)
    $arquivo = str_pad($arquivo, 26, "0");

    // CAMPO: Valor / percentual
    // POSIÇÃO: 027 a 041
    // PICTURE: 9(15)
    $arquivo = str_pad($arquivo, 41, "0");

    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 042 - 065
    // PICTURE: 024
    $arquivo = str_pad($arquivo, 65);

    // CAMPO: Código da multa
    // POSIÇÃO: 066 a 066
    // PICTURE: 9(1)    1- Valor Fixo 2- Percentual
    $vrpemulbrec = $valorpemulbrec;
    if($vrpemulbrec == 0)
        $arquivo .= Util::campoCnabValido(self::CODIGO_MULTA_1);
    else
        $arquivo .= Util::campoCnabValido(self::CODIGO_MULTA_2);

    // CAMPO: Data da multa
    // POSIÇÃO: 067 a 074
    // PICTURE: 9(8)
    // ESTA DATA NÃO PODE FICAR ZERADA.
    $now = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
    $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
    $arquivo .= Util::campoCnabValido($data);

    // CAMPO: Valor / percentual
    // POSIÇÃO: 075 a 089
    // PICTURE: 9(15)
    $vrmultabrec = $valormultabrec;
    if (($vrpemulbrec == 0) or ((bool) $vrpemulbrec === false)){
        if (($vrmultabrec == 0) or ((bool) $vrmultabrec === false)){
            $arquivo = str_pad($arquivo, 89, "0");
        }
        else{
            $vrmultabrec = $vrmultabrec * 100;
            $arquivo .= Util::campoCnabValido(str_pad($vrmultabrec, 15, '0', STR_PAD_LEFT));
        }
    }
    else{
        $vrpemulbrec = $vrpemulbrec * 100;
        $arquivo .= Util::campoCnabValido(str_pad($vrpemulbrec, 15, '0', STR_PAD_LEFT));
    }

    // CAMPO: Informação do banco ao sacado
    // POSIÇÃO: 090 a 099
    // PICTURE: 9(10)
    $arquivo = str_pad($arquivo, 99);

    // CAMPO: Mensagem 3
    // POSIÇÃO: 100 a 139
    // PICTURE: X(40)
    /** @var $boletreceber Boletreceber */
    $boletreceber = Boletreceber::findOneByNrorgAndNrlanbolrec($nrorg, $nrlanbolrec);
    if((bool)$boletreceber->getTxmensbolbol() === false){
        $arquivo = str_pad($arquivo, 139);
    }	
    else{
        $arquivo .= Util::campoCnabValido($boletreceber->getTxmensbolbol());
    }

    // CAMPO: Mensagem 4
    // POSIÇÃO: 140 a 179
    // PICTURE: X(40)
    $arquivo = str_pad($arquivo, 179);

    // CAMPO: Reservado
    // POSIÇÃO: 180 - 240
    // PICTURE: 061
    $arquivo = str_pad($arquivo, 240) . "\r\n";

    fprintf($file, "%s", $arquivo);
    fclose($file);
}

public function GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco){

    $nmarquivo = $caminho.$nmarquivo;
    $file = fopen($nmarquivo, "a+");
    tk_set_env_var('REGISTROS', tk_get_env_var('REGISTROS') + 1);
    tk_set_env_var('LOTES', tk_get_env_var('LOTES') + 1);

    // CAMPO: Código do Banco
    // POSIÇÃO: 001 a 003
    // PICTURE: 9(3)
    /** @var $banco Banco */
    $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
    $arquivo = $banco->getCdbanccnab();

    // CAMPO: Lote de Serviço
    // POSIÇÃO: 004 a 007
    // PICTURE: 9(4)
    $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_2);

    // CAMPO: Identificação do Arquivo
    // POSIÇÃO: 008 a 008
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_ARQUIVO_TRAILLER_LOTE);

    // CAMPO: Uso exclusivo FEBRABAN
    // POSIÇÃO: 009 a 017
    // PICTURE: X(9)
    $arquivo = str_pad($arquivo, 17);

    // CAMPO: Quantidade de registros do arquivo
    // POSIÇÃO: 018 a 023
    // PICTURE: 9(6)
    $arquivo .= Util::campoCnabValido(str_pad(tk_get_env_var('REGISTROS') - 1, 6, "0", STR_PAD_LEFT));

    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 024 - 240
    // PICTURE: 9(92)
    $arquivo = str_pad($arquivo, 240) . "\r\n";

    fprintf($file, "%s", $arquivo);
    fclose($file);
}

public function GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco){

    $nmarquivo = $caminho.$nmarquivo;
    $file = fopen($nmarquivo, "a+");
    tk_set_env_var('REGISTROS', tk_get_env_var('REGISTROS') + 1);

    // CAMPO: Código do Banco
    // POSIÇÃO: 001 a 003
    // PICTURE: 9(3)
    /**@var $banco Banco */
    $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
    $arquivo = $banco->getCdbanccnab();

    // CAMPO: Lote de Serviço
    // POSIÇÃO: 004 a 007
    // PICTURE: 9(4)
    $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_TRAILLER);

    // CAMPO: Identificação do Arquivo
    // POSIÇÃO: 008 a 008
    // PICTURE: 9(1)
    $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_ARQUIVO_TRAILLER);

    // CAMPO: Uso exclusivo FEBRABAN
    // POSIÇÃO: 009 a 017
    // PICTURE: X(9)
    $arquivo = str_pad($arquivo, 17);

    // CAMPO: Quantidade de lotes do arquivo
    // POSIÇÃO: 018 a 023
    // PICTURE: 9(6)
    $arquivo .= Util::campoCnabValido(str_pad(tk_get_env_var('LOTES'), 6, "0", STR_PAD_LEFT));

    // CAMPO: Quantidade de registros do arquivo
    // POSIÇÃO: 024 a 029
    // PICTURE: 9(6)
    $arquivo .= Util::campoCnabValido(str_pad(tk_get_env_var('REGISTROS'), 6, "0", STR_PAD_LEFT));

    // CAMPO: Reservado (uso Banco)
    // POSIÇÃO: 030 - 240
    // PICTURE: 211
    $arquivo = str_pad($arquivo, 240) . "\r\n";

    fprintf($file, "%s", $arquivo);
    fclose($file);
    
    }

    public function geraTransacao($caminho, $nmarquivo, $cdempresa, $cdcartcobbol, $cdagencia, $cdcontcorr, $cdfilial,
                                           $nrlanbolrec, $vrpemobrec, $idimprboleto, $cdnosnumrec, $dtvenbolrec, $cdins1cobbol,
                                           $qtdiaprotbol, $vrmordiabrec, $vrfinalbrec, $idopecnabrec, $vrpemulbrec, $idtpijurclie,
                                           $nrinsjurclie, $cdcliente, $cdins2cobbol){}
}